import { AstNode } from "./core/AstNode";
export class Class extends AstNode {
    name;
    extends_;
    decorators;
    fields = [];
    docs;
    statements = [];
    constructor({ docs, name, extends_, decorators }) {
        super();
        this.name = name;
        this.extends_ = extends_ ?? [];
        this.decorators = decorators ?? [];
        this.docs = docs;
        this.extends_.forEach((parentClassReference) => {
            this.inheritReferences(parentClassReference);
        });
        this.decorators.forEach((decorator) => {
            this.inheritReferences(decorator);
        });
    }
    write(writer) {
        this.decorators.forEach((decorator) => {
            decorator.write(writer);
        });
        writer.write(`class ${this.name}`);
        if (this.extends_.length > 0) {
            writer.write("(");
            this.extends_.forEach((parentClassReference, index) => {
                if (index > 0) {
                    writer.write(", ");
                }
                parentClassReference.write(writer);
            });
            writer.write(")");
        }
        writer.write(":");
        writer.newLine();
        if (this.docs != null) {
            writer.write('"""');
            writer.write(this.docs);
            writer.write('"""');
        }
        writer.writeNewLineIfLastLineNot();
        writer.indent();
        this.fields.forEach((field) => {
            field.write(writer);
            writer.writeNewLineIfLastLineNot();
        });
        writer.dedent();
        writer.indent();
        if (this.statements.length) {
            this.writeStatements({ writer });
        }
        else {
            writer.write("pass");
        }
        writer.dedent();
    }
    add(statement) {
        this.statements.push(statement);
        this.inheritReferences(statement);
    }
    writeStatements({ writer }) {
        this.statements.forEach((statement, index) => {
            statement.write(writer);
            writer.writeNewLineIfLastLineNot();
        });
    }
    addField(field) {
        this.add(field);
    }
}
