import { AstNode } from "./core/AstNode";
export class Reference extends AstNode {
    name;
    modulePath;
    genericTypes;
    alias;
    attribute;
    constructor({ name, modulePath, genericTypes, alias, attribute }) {
        super();
        this.name = name;
        this.modulePath = modulePath ?? [];
        this.genericTypes = genericTypes ?? [];
        this.alias = alias;
        this.attribute = attribute ?? [];
        this.references.push(this);
        this.genericTypes.forEach((genericType) => {
            this.inheritReferences(genericType);
        });
    }
    write(writer) {
        const nameOverride = writer.getRefNameOverride(this);
        writer.write(nameOverride.name);
        if (this.genericTypes.length > 0) {
            writer.write("[");
            this.genericTypes.forEach((genericType, index) => {
                if (index > 0) {
                    writer.write(", ");
                }
                genericType.write(writer);
            });
            writer.write("]");
        }
        if (this.attribute.length > 0) {
            writer.write(".");
            this.attribute.forEach((attr, index) => {
                if (index > 0) {
                    writer.write(".");
                }
                writer.write(attr);
            });
        }
    }
    getFullyQualifiedPath() {
        return this.modulePath.join(".");
    }
    getFullyQualifiedModulePath() {
        return `${this.getFullyQualifiedPath()}.${this.name}`;
    }
}
